<?xml version="1.0" encoding="UTF-8"?><?xar XSLT?>
<xsl:stylesheet version="2.0"
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:ps="http://www.pageseeder.com/editing/2.0"
                xmlns:woext="java:com.weborganic.web.XTemplateExtensions"
                extension-element-prefixes="woext">

  <xsl:import href="Utilities.xsl" />

  <xsl:param name="anchorprefix" select="''" />
  <xsl:param name="servletprefix" select="'dunno'" />
  <xsl:param name="xsl-sourceuri" select="''" />
  <xsl:param name="xsl-sourcefrag" select="''" />
  <xsl:param name="xsl-sourcetitle" select="''" />
  <xsl:param name="xsl-targettitle" select="''" />
  <xsl:param name="ps-unresolved-xrefs" select="'false'" />
  <xsl:param name="wo-request.ps-release" select="'ps-default-value'" />
  <xsl:param name="wo-request.ps-releasestatus" select="'ps-default-value'" />
  
  <!-- changed from '/root' to '//root' to make the numbering work with layout manager XML -->
  <xsl:variable name="root" select="//root"/>
  <xsl:variable name="default-scope" select="'Default'" />
  
  <xsl:variable name="unresolved-xrefs" select="$ps-unresolved-xrefs = 'true'" />
 
 <xsl:variable name="titles">
  <xsl:copy-of select="$root//ps:toc/*[starts-with(name(),'ps:level')]
                                      [not(ancestor::ps:documentation[ps:status='Compare'])]"/>
 </xsl:variable>

 <xsl:template name="current-title">
  <xsl:param name="elem" />
  <xsl:param name="include-nb" tunnel="yes" select="true()" />
  <xsl:param name="include-text" tunnel="yes" select="true()" />
  <xsl:choose>
    <xsl:when test="$include-nb = true() and $include-text = true()">
      <xsl:value-of select="if ($elem/@label) then concat($elem/@label, $elem) else $titles/*[@id = ps:generate-toc-id($elem)]/concat(@label, .)" />
    </xsl:when>
    <xsl:when test="$include-nb = true()">
      <xsl:variable name="nb" select="if ($elem/@label) then $elem/@label else $titles/*[@id = ps:generate-toc-id($elem)]/@label" />
      <xsl:value-of select="replace($nb, '\.$', '')" />
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="$titles/*[@id = ps:generate-toc-id($elem)]" />
    </xsl:otherwise>
  </xsl:choose>
 </xsl:template>

  <!--
    Generates insert XRef reference function

    @param frag         target fragment 
    @param uriid        id of target URI
    @param rte          name of rich text editor variable
    @param siteprefix   PageSeeder site prefix
    @param section      auto section: if 'true' allow automatic title update for sections
                        (default is 'true')
    @param title        the XRef title to be displayed
    @param ref-text     the function text to display (e.g. 'Reference' or 'Reference Document')
    @param manual       manual title: if 'true' default to manual title update
                        (default is 'false')
  
    @return the in-line HTML for the function
  -->
  <xsl:template name="accept-xref">
    <xsl:param name="frag" select="'default'" />
    <xsl:param name="uriid" />
    <xsl:param name="rte" select= "'rte1'" />
    <xsl:param name="siteprefix" select="'undefined'"/>
    <xsl:param name="allow-auto" select="'true'"/> <!-- not used -->
    <xsl:param name="title" />
    <xsl:param name="ref-text" select="'Reference'" />
    <xsl:param name="manual" select="'false'" />
    
    <xsl:variable name="xsl-params">        
        <xsl:text>&amp;sourceuri=</xsl:text>
        <xsl:choose><xsl:when test="$xsl-sourceuri != ''"><xsl:value-of select="$xsl-sourceuri"/></xsl:when>
    <xsl:otherwise><xsl:value-of select="/root/wo-request/xsl-sourceuri"/></xsl:otherwise></xsl:choose>
        <xsl:text>&amp;sourcefrag=</xsl:text>
        <xsl:choose><xsl:when test="$xsl-sourcefrag != ''"><xsl:value-of select="$xsl-sourcefrag"/></xsl:when>
    <xsl:otherwise><xsl:value-of select="/root/wo-request/xsl-sourcefrag"/></xsl:otherwise></xsl:choose>
        <xsl:text>&amp;sourcetitle=</xsl:text>
        <xsl:choose><xsl:when test="$xsl-sourcetitle != ''"><xsl:value-of select="encode-for-uri($xsl-sourcetitle)"/></xsl:when>
        <xsl:otherwise><xsl:value-of select="encode-for-uri(/root/wo-request/xsl-sourcetitle)"/></xsl:otherwise></xsl:choose>
    </xsl:variable>
  <xsl:variable name="targettitle" select="if ($xsl-targettitle = '') then $title else $xsl-targettitle" />
  <xsl:variable name="display" select="if ($xsl-targettitle != '' or $manual = 'true') then '&amp;display=manual' else ''" />
    <P ALIGN="RIGHT">
    <A HREF="javascript:parent.window.close()">
        Cancel <IMG SRC="{$siteprefix}/weborganic/common/cross.gif" HEIGHT="8" WIDTH="10" BORDER="0" /></A>
    <xsl:text>&#160;</xsl:text>
    <!--A HREF="javascript:parent.window.opener.insertXref('{$rte}', '{$title}', '{$uriid}', '{$frag}', false, {$section});"-->
    <A HREF="{$servletprefix}/com.pageseeder.xref.InsertXRef?rte={$rte}&amp;targettitle={encode-for-uri($targettitle)}&amp;targeturi={$uriid}&amp;targetfrag={$frag}{$display}{$xsl-params}">
        <xsl:value-of select="$ref-text" /> <IMG SRC="{$siteprefix}/weborganic/common/seed.gif" HEIGHT="12" WIDTH="15" BORDER="0" ALIGN="top" />
    </A>
    </P>
  </xsl:template>
  
 <!-- gets possible child elements of an xs:element with @name -->
 <xsl:function name="ps:generate-toc-id">
    <xsl:param name="elem" />   
    <xsl:value-of select="$elem/@id"/>
 </xsl:function>

 <xsl:template name="part-title">
  <xsl:param name="part" />
  <xsl:choose>
   <xsl:when test="$part/descendant::*[@numbered = 'true'][translate(normalize-space(.), ' \t&#xa0;', '') != '']">
    <xsl:call-template name="current-title">
     <xsl:with-param name="elem" select="$part/descendant::*[@numbered = 'true'][translate(normalize-space(.), ' \t&#xa0;', '') != ''][1]" />
    </xsl:call-template>
   </xsl:when>
   <xsl:when test="$part/preceding::ps:part">
    <xsl:call-template name="part-last-title">
     <xsl:with-param name="part" select="$part/preceding::ps:part[1]" />
    </xsl:call-template>
   </xsl:when>
   <xsl:otherwise>0</xsl:otherwise>
  </xsl:choose>
 </xsl:template>

 <xsl:template match="br" mode="compute-title" priority="-1">
  <xsl:apply-templates mode="compute-title" />
 </xsl:template>
 <xsl:template match="br" mode="compute-title">
  <xsl:text> </xsl:text>
 </xsl:template>
  
 <xsl:template name="part-unnumbered-title">
  <xsl:param name="part" />
  <xsl:choose>
   <!-- first check for headings in the current section -->
   <xsl:when test="$part/descendant::*[contains(local-name(), 'heading')][translate(normalize-space(.), ' \t&#xa0;', '') != '']">
    <xsl:apply-templates select="$part/descendant::*[contains(local-name(), 'heading')][translate(normalize-space(.), ' \t&#xa0;', '') != ''][1]" mode="compute-title" />
   </xsl:when>
   <!-- then check for headings in previous parts in the same section -->
   <xsl:when test="$part/preceding-sibling::ps:part/descendant::*[contains(local-name(), 'heading')][translate(normalize-space(.), ' \t&#xa0;', '') != '']">
    <xsl:apply-templates select="$part/(preceding-sibling::ps:part/descendant::*[contains(local-name(), 'heading')][translate(normalize-space(.), ' \t&#xa0;', '') != ''])[last()]" mode="compute-title" />
   </xsl:when>
   <!-- ok, check for section title -->
   <xsl:when test="$part/ancestor::section/title">
    <xsl:apply-templates select="$part/ancestor::section/title" mode="compute-title" />
   </xsl:when>
   <!-- headings in other sections? -->
   <xsl:when test="$part/preceding::ps:part/descendant::*[contains(local-name(), 'heading')][translate(normalize-space(.), ' \t&#xa0;', '') != '']">
    <xsl:apply-templates select="$part/(preceding::ps:part/descendant::*[contains(local-name(), 'heading')][translate(normalize-space(.), ' \t&#xa0;', '') != ''])[last()]" mode="compute-title" />
   </xsl:when>
   <!-- try title of other sections -->
   <xsl:when test="$part/preceding::section/title">
    <xsl:apply-templates select="$part/(preceding::section/title)[last()]" mode="compute-title" />
   </xsl:when>
   <!-- then root title -->
   <xsl:when test="$part/ancestor::root/title">
    <xsl:apply-templates select="$part/ancestor::root/title" mode="compute-title" />
   </xsl:when>
   <!-- give up -->
   <xsl:otherwise>0</xsl:otherwise>
  </xsl:choose>
 </xsl:template>
   
 <xsl:template name="part-last-title">
  <xsl:param name="part" />
  <xsl:choose>
   <xsl:when test="$part/descendant::*[@numbered = 'true'][translate(normalize-space(.), ' \t&#xa0;', '') != '']">
    <xsl:call-template name="current-title">
     <xsl:with-param name="elem" select="$part/descendant::*[@numbered = 'true'][translate(normalize-space(.), ' \t&#xa0;', '') != ''][last()]" />
    </xsl:call-template>
   </xsl:when>
   <xsl:when test="$part/preceding::ps:part">
    <xsl:call-template name="part-last-title">
     <xsl:with-param name="part" select="$part/preceding::ps:part[1]" />
    </xsl:call-template>
   </xsl:when>
   <xsl:otherwise>0</xsl:otherwise>
  </xsl:choose>
 </xsl:template>
   
 <xsl:template name="update-title">
  <xsl:param name="title" />
  <xsl:param name="level" />
  <xsl:variable name="levels" select="tokenize($title, '\.')" />
  <xsl:for-each select="$levels">
   <xsl:choose>
    <xsl:when test="position() = number($level)"><xsl:value-of select="number(.) + 1" /></xsl:when>
    <xsl:when test="position() &gt; number($level)"></xsl:when>
    <xsl:otherwise><xsl:value-of select="." /></xsl:otherwise>
   </xsl:choose>
   <xsl:if test="position() != last() and position() &lt; number($level)">.</xsl:if>
  </xsl:for-each>
  <xsl:if test="number($level) &gt; count($levels)">
   <xsl:call-template name="up-title-to">
    <xsl:with-param name="from" select="count($levels)" />
    <xsl:with-param name="to" select="number($level)" />
   </xsl:call-template>
  </xsl:if>
 </xsl:template>
 
 <xsl:template name="up-title-to">
  <xsl:param name="from" />
  <xsl:param name="to" />
  <xsl:choose>
   <xsl:when test="$to = $from + 1">
    <xsl:text>.1</xsl:text>
   </xsl:when>
   <xsl:when test="$to &gt; $from + 1">
    <xsl:text>.0</xsl:text>
    <xsl:call-template name="up-title-to">
     <xsl:with-param name="from" select="$from + 1" />
     <xsl:with-param name="to" select="$to" />
    </xsl:call-template>
   </xsl:when>
  </xsl:choose>
 </xsl:template>
 
 <xsl:template name="get-level-of-element">
  <xsl:param name="elem" />
  <xsl:choose>
   <xsl:when test="string-length(substring-after(name($elem), 'heading')) = 1">
    <xsl:value-of select="number(substring-after(name($elem), 'heading'))" />
   </xsl:when>
   <xsl:otherwise>
    <!-- ok, this is not a heading, then take previous heading level and increase it -->
    <xsl:variable name="previous" select="$elem/preceding::*[@numbered='true' and
                    string-length(substring-after(name(), 'heading')) = 1]" />
    <!--
    <xsl:variable name="previous" select="$elem/preceding::*[@numbered='true' and
                    ancestor::ps:documentation/ps:scope = $default-scope and
                    string-length(substring-after(name(), 'heading')) = 1]" />
    -->
    <xsl:choose>
     <xsl:when test="$previous">
      <xsl:value-of select="1+number(substring-after(name($previous[last()]), 'heading'))" />
     </xsl:when>
     <xsl:otherwise>2</xsl:otherwise>
    </xsl:choose>
   </xsl:otherwise>
  </xsl:choose>
 </xsl:template>
 
  <!-- Turn attribute uriid into actual href attribute -->
  <xsl:template match="xref | blockXref" mode="merge-copy">
    <xsl:param name="xlinksforlocator" select="$root/documentxlinks/locators/locator/xlinksforlocator" tunnel="yes"/>
    <xsl:param name="show-unresolved" select="$unresolved-xrefs" tunnel="yes"/>
    <!-- retrieve xlink -->
    <xsl:variable name="frag" select="@frag" />
    <xsl:variable name="uriid" select="@uriid" />
    <xsl:variable name="xref-role">
      <xsl:choose>
        <xsl:when test="@manual='true'">XRef-manual</xsl:when><!-- TODO: remove, only here for backwards compatibility -->
        <xsl:when test="@display"><xsl:value-of select="concat('XRef-',@display)"/></xsl:when>
        <xsl:otherwise>XRef</xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="lfx" select="$xlinksforlocator
      /locatorforxlink/xlink[contentrole=$xref-role or ($show-unresolved and contentrole=concat('archive-',$xref-role))]
      /locatorsforxlink/locatorforxlink[role='xref-link' and locator/fragment=$frag and locator/uris/uri/id=$uriid]" />
    <!-- only include xref if there is a corresponding xlink -->
    <xsl:if test="$lfx or $show-unresolved">
      <xsl:copy>
        <xsl:copy-of select="@labels | @uriid | @frag | @title" />
        <xsl:variable name="display" select="if (@display and @display!='') then @display else if (@manual = 'true') then 'manual' else 'document'" />
        <xsl:attribute name="display" select="$display"/>
        <!-- re-compute type from xlink for backward compatibility -->
        <xsl:variable name="type">
          <xsl:choose>
            <xsl:when test="@type and @type!=''"><xsl:value-of select="@type" /></xsl:when>
            <xsl:when test="$lfx[1]/label"><xsl:value-of select="$lfx[1]/label" /></xsl:when>
            <xsl:otherwise>None</xsl:otherwise>
          </xsl:choose>
        </xsl:variable>
        <xsl:attribute name="type" select="$type"/>
        <xsl:variable name="reverselink" select="if (@reverselink) then @reverselink else @doublelink" />
        <xsl:attribute name="reverselink" select="boolean($reverselink = 'true')"/>
        <xsl:if test="$reverselink = 'true'">
          <xsl:attribute name="reversetitle" select="if (@reversetitle) then @reversetitle else @doublelinktitle"/>
          <xsl:attribute name="reversetype">
            <xsl:choose>
              <xsl:when test="@reversetype and @reversetype!=''"><xsl:value-of select="@reversetype" /></xsl:when>
              <!-- Set reverse type for backward compatibility -->
              <xsl:when test="$type='Effects'">Depends On</xsl:when>
              <xsl:when test="$type='Depends On'">Effects</xsl:when>
              <xsl:otherwise>None</xsl:otherwise>
            </xsl:choose>
          </xsl:attribute>
        </xsl:if>
        <xsl:variable name="href">
          <xsl:choose>
            <xsl:when test="$lfx">
              <xsl:call-template name="get-href-for-xref">
                <xsl:with-param name="source-uriid" select="$root/documentxlinks/id" />
                <xsl:with-param name="target-uri" select="$lfx[1]/locator/uris/uri" />
                <xsl:with-param name="target-path" select="$lfx[1]/locator/uris/uri/path" />
                <xsl:with-param name="frag" select="$frag" />
              </xsl:call-template>
            </xsl:when>
            <xsl:when test="@href"><xsl:value-of select="@href" /></xsl:when>
          </xsl:choose>
        </xsl:variable>
        <xsl:if test="string($href) != ''"><xsl:attribute name="href" select="$href" /></xsl:if>
        <!--  add an attribute for target document title (used in XRef explorer -->
        <xsl:attribute name="urititle">
          <xsl:call-template name="get-uri-name">
            <xsl:with-param name="uri" select="$lfx[1]/locator/uris/uri" />
          </xsl:call-template>
        </xsl:attribute>
        <!-- docid attribute needed for show-unresolved -->
        <xsl:choose>
          <xsl:when test="$lfx[1]/locator/uris/uri/datetime">
            <xsl:attribute name="docid" select="$lfx[1]/locator/uris/uri/datetime"/>
          </xsl:when>
          <xsl:when test="$show-unresolved and @docid">
            <xsl:attribute name="docid" select="@docid"/>
          </xsl:when>
        </xsl:choose>
        <!--  Value -->
        <xsl:choose>
          <xsl:when test="$display='document' and $lfx">
            <xsl:call-template name="get-uri-name">
              <xsl:with-param name="uri" select="$lfx[1]/locator/uris/uri" />
            </xsl:call-template>
          </xsl:when>
          <xsl:when test="$display='document+manual' and $lfx">
            <xsl:call-template name="get-uri-name">
              <xsl:with-param name="uri" select="$lfx[1]/locator/uris/uri" />
            </xsl:call-template>
            <xsl:text>: </xsl:text>
            <xsl:value-of select="@title" />
          </xsl:when>
          <xsl:when test="($display='fragment' or $display='document+fragment') and $lfx"> <!-- backwards compatible -->
            <xsl:call-template name="get-uri-name">
              <xsl:with-param name="uri" select="$lfx[1]/locator/uris/uri" />
            </xsl:call-template>
            <xsl:text>: </xsl:text>
            <xsl:value-of select="$frag"/>
          </xsl:when>
          <xsl:when test="$display='manual' and @title != ''"><xsl:value-of select="@title" /></xsl:when>
          <xsl:when test="$lfx">
            <!-- Display document name for old autonumber labels -->
            <xsl:call-template name="get-uri-name">
              <xsl:with-param name="uri" select="$lfx[1]/locator/uris/uri" />
            </xsl:call-template>
          </xsl:when>
          <xsl:otherwise><xsl:value-of select="@title" /></xsl:otherwise>
        </xsl:choose>
      </xsl:copy>
    </xsl:if>
  </xsl:template>
 
    
  <xsl:template name="reverse-xrefs">
    <xsl:param name="fragment" select="'default'" />
    <xsl:param name="xrefs" />
    <xsl:param name="uriid" />

    <xsl:if test="$xrefs/xlink">
      <!-- Double link display -->
      <div class="reverse-xrefs">
        <button type="button" class="button-xref collapse" id="button-xref{$fragment}" title=""
            onclick="javascript:toggleLinks('xref{$fragment}');">
          <span>Links:</span>
        </button>
        <span class="xref-icon">&#160;</span>
        <div class="xreflist" id="xref{$fragment}">
          <ul>
            <xsl:for-each select="$xrefs/xlink/locatorsforxlink/locatorforxlink[starts-with(role, 'xref-source-')]">
              <xsl:variable name="urititle">
                <xsl:call-template name="get-uri-name">
                  <xsl:with-param name="uri" select=".//uri" />
                </xsl:call-template>
              </xsl:variable>
                  <xsl:variable name="href">
                    <xsl:call-template name="get-href-for-xref">
                        <xsl:with-param name="source-uriid" select="$uriid" />
                        <xsl:with-param name="target-uri" select=".//uri" />
                        <xsl:with-param name="target-path" select=".//uri/path" />
                        <xsl:with-param name="frag" select="locator/fragment" />
                    </xsl:call-template>
                  </xsl:variable>
              <li><a href="{$href}" target="_top"><xsl:value-of
                    select="if (title != '') then concat($urititle, ': ', title) else $urititle" /></a></li>
            </xsl:for-each>
          </ul>
        </div>
      </div>
      <!-- done -->
    </xsl:if>
  </xsl:template>

  <xsl:template name="get-href-for-xref">
    <xsl:param name="source-uriid" />
    <xsl:param name="target-uri" />
    <xsl:param name="target-path" />
    <xsl:param name="frag" />
    
    <xsl:choose>
        <!-- if current file, just hash -->
        <xsl:when test="$target-uri/id = $source-uriid and $frag = 'default'">#top</xsl:when>
        <xsl:when test="$target-uri/id = $source-uriid">
            <xsl:value-of select="concat('#', $anchorprefix, 'Frag', $frag)" />
        </xsl:when>
    <xsl:when test="$frag = 'default' and $wo-request.ps-release != 'ps-default-value' and $wo-request.ps-releasestatus != 'ps-default-value'">
      <xsl:value-of select="concat($target-path, '?ps-release=', $wo-request.ps-release, '&amp;ps-releasestatus=', $wo-request.ps-releasestatus)"/>
    </xsl:when>
        <xsl:when test="$frag = 'default' and $wo-request.ps-release != 'ps-default-value'">
            <xsl:value-of select="concat($target-path, '?ps-release=', $wo-request.ps-release)"/>
        </xsl:when>
    <xsl:when test="$frag = 'default' and $wo-request.ps-releasestatus != 'ps-default-value'">
      <xsl:value-of select="concat($target-path, '?ps-releasestatus=', $wo-request.ps-releasestatus)"/>
    </xsl:when>
    <xsl:when test="$frag = 'default'">
      <xsl:value-of select="$target-path"/>
    </xsl:when>
    <xsl:when test="$wo-request.ps-release != 'ps-default-value' and $wo-request.ps-releasestatus != 'ps-default-value'">
      <xsl:value-of select="concat($target-path, '?ps-release=', $wo-request.ps-release, '&amp;ps-releasestatus=', $wo-request.ps-releasestatus, '&amp;frag=', $frag, '#', $anchorprefix, 'Frag', $frag)" />
    </xsl:when>
    <xsl:when test="$wo-request.ps-release != 'ps-default-value'">
      <xsl:value-of select="concat($target-path, '?ps-release=', $wo-request.ps-release, '&amp;frag=', $frag, '#', $anchorprefix, 'Frag', $frag)" />
    </xsl:when>
    <xsl:when test="$wo-request.ps-releasestatus != 'ps-default-value'">
      <xsl:value-of select="concat($target-path, '?ps-releasestatus=', $wo-request.ps-releasestatus, '&amp;frag=', $frag, '#', $anchorprefix, 'Frag', $frag)" />
    </xsl:when>
        <xsl:otherwise>
            <xsl:value-of select="concat($target-path, '?frag=', $frag, '#', $anchorprefix, 'Frag', $frag)" />
        </xsl:otherwise>
    </xsl:choose> 
  </xsl:template>
   
</xsl:stylesheet>