<?xml version="1.0"?>
<xsl:stylesheet version="2.0"
                xmlns="http://www.w3.org/1999/xhtml"
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xs="http://www.w3.org/2001/XMLSchema"
                xmlns:psf="http://www.pageseeder.com/function"
                xmlns:system="java:java.lang.System"
                xmlns:dec="java:java.net.URLDecoder"
                xmlns:enc="java:java.net.URLEncoder"
                exclude-result-prefixes="xs psf dec enc system">
<!--
A collection of general utility functions for the simple interface.

@author  Christophe Lauret
@version 20/11/08
@since   v33305

Copyright (C) 2007 Weborganic Systems Pty. Ltd.
-->

<!-- Import block functions -->
<xsl:import href="../Common/block-functions.xsl" />

<!-- Group name -->
<xsl:param name="group.name" />
<!-- Page path -->
<xsl:param name="page.path" />
<!-- Page query string -->
<xsl:param name="page.query-string" />
<!-- Site prefix -->
<xsl:param name="wo-constant.siteprefix" />

<!--
Toggle on or off of the Content
  
This function use *id* as a identification for block. Do nothing if the *id* is invalid.
-->
<xsl:function name="psf:toggle-content">
  <!-- The id of the block the button is used for -->
  <xsl:param name="id" as="xs:string"/>
  <a href="#">
    <xsl:choose>
      <xsl:when test="string-length($id) gt 0">
        <xsl:attribute name="class">button</xsl:attribute>
        <xsl:attribute name="id">button-<xsl:value-of select="$id"/></xsl:attribute>
        <xsl:attribute name="onclick">JavaScript:return PAGESEEDER.Utils.toggleContent('<xsl:value-of select="$id"/>');</xsl:attribute>
        <xsl:attribute name="title">Expand / Collapse</xsl:attribute>
      </xsl:when>
      <xsl:otherwise>
        <xsl:attribute name="class">button collapse</xsl:attribute>
        <xsl:attribute name="onclick">JavaScript:return false;</xsl:attribute>
        <xsl:attribute name="title">This item cannot be collapsed</xsl:attribute>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:text>&#160;</xsl:text>
  </a>
</xsl:function>

<!--
  Matches any part of the text which contains a string starting with
  'http://' or 'https://' and create a link.

  @param text the text node to work from.
  @return a sequence of the text nodes and links.
-->
<xsl:function name="psf:linkify">
  <xsl:param name="text" />
  <xsl:for-each select="$text">
    <xsl:analyze-string select="." regex="http[s]?://[^\s]+">
      <xsl:non-matching-substring>
        <xsl:value-of select="."/>
      </xsl:non-matching-substring>
      <xsl:matching-substring>
        <a href="{.}"><xsl:value-of select="."/></a>
      </xsl:matching-substring>
    </xsl:analyze-string>
  </xsl:for-each>
</xsl:function>

<!--
  Ouput the heading link for toggle content
-->
<xsl:function name="psf:toggle-content-with-heading">
  <!-- The id of the block the button is used for. -->
  <xsl:param name="id" 		as="xs:string"/>
  <!-- The heading title -->
  <xsl:param name="heading" as="xs:string"/>
  <!-- The total number of the current categories -->
  <xsl:param name="count" 	as="xs:integer"/>
  <a href="#">
    <xsl:choose>
      <xsl:when test="string-length($id) gt 0">
        <xsl:attribute name="class">button</xsl:attribute>
        <xsl:attribute name="id">button-<xsl:value-of select="$id"/></xsl:attribute>
        <xsl:attribute name="onclick">JavaScript:return PAGESEEDER.Utils.toggleContent('<xsl:value-of select="$id"/>');</xsl:attribute>
        <xsl:attribute name="title">Expand / Collapse</xsl:attribute>
      </xsl:when>
      <xsl:otherwise>
        <xsl:attribute name="class">button collapse</xsl:attribute>
        <xsl:attribute name="onclick">JavaScript:return false;</xsl:attribute>
        <xsl:attribute name="title">This item cannot be collapsed</xsl:attribute>
      </xsl:otherwise>
    </xsl:choose>
    <span class="button">&#160;</span>
    <span><xsl:value-of select="$heading"/> (<xsl:value-of select="$count"/>)</span>
  </a>
</xsl:function>

<!--
  Replaces the newlines \n by <br/> tags.
-->
<xsl:function name="psf:replace-newlines">  
  <!-- The string process. -->
  <xsl:param name="text"/>
  
  <xsl:if test="not(empty($text))">
    <xsl:analyze-string select="$text" regex="\n">
      <xsl:non-matching-substring>
        <xsl:value-of select="."/>
      </xsl:non-matching-substring>
      <xsl:matching-substring>
        <br/>
      </xsl:matching-substring>
    </xsl:analyze-string>
  </xsl:if>
</xsl:function>

<!--
  Replaces the newlines \n by <br/> tags.
-->
<xsl:function name="psf:replace-newlines-linkify">  
  <!-- The string process. -->
  <xsl:param name="text"/>
  <xsl:for-each select="$text">
	  <xsl:analyze-string select="." regex="\n">
	    <xsl:non-matching-substring>
	      <xsl:copy-of select="psf:linkify(.)"/>
	    </xsl:non-matching-substring>
	    <xsl:matching-substring>
	      <br/>
	    </xsl:matching-substring>
	  </xsl:analyze-string>
  </xsl:for-each>
</xsl:function>

<!--
  Wraps the keyword in a <em class="keyword"> ... </em> tag.
-->
<xsl:function name="psf:highlight-keyword">
  <!-- The text process. -->
  <xsl:param name="text"/>
  <!-- The keyword process. -->
  <xsl:param name="keyword"/>
  <xsl:variable name="quotes" select="matches($keyword, '^&quot;.*?&quot;$')" />
  <xsl:variable name="keyw" select="if ($quotes) then replace($keyword, '^.|.$', '') else $keyword" />
  <xsl:variable name="regex" select="if (contains($keyw, ' ') and not($quotes)) then string-join(tokenize($keyw, ' '), '|') else $keyw" />
  <xsl:choose>
    <xsl:when test="$keyword">
      <xsl:analyze-string select="$text" regex="{$regex}" flags="i">
        <xsl:matching-substring>
          <em class="keyword"><xsl:value-of select="."/></em>
        </xsl:matching-substring>
        <xsl:non-matching-substring>
          <xsl:value-of select="."/>
        </xsl:non-matching-substring>
      </xsl:analyze-string>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="$text"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:function>

<!--
  Displays the specified value if the condition is true, otherwise return empty.
-->
<xsl:function name="psf:empty">
  <!-- The condition to check. -->
  <xsl:param name="condition"/>
  <!-- The value when the condition is not empty. -->
  <xsl:param name="value"/>
  <xsl:value-of select="if ($condition) then string($value) else 'empty'"/>
</xsl:function>

<!--
  Returns ' selected' if the condition is <code>true</code> otherwise returns an empty string.
-->
<xsl:function name="psf:selected-if">
  <!-- The condition to check. -->
  <xsl:param name="condition" as="xs:boolean"/>
  <xsl:value-of select="if ($condition) then ' selected' else ''"/>
</xsl:function>

<!--
  Displays the specified value if it exists otherwise returns '&#45;&#45;'.
-->
<xsl:function name="psf:display">
  <!-- The value when the condition is not empty. -->
  <xsl:param name="value"/>
  <xsl:value-of select="if ($value) then $value else '&#45;&#45;'"/>
</xsl:function>

<!--
  Returns the full name of the member, by concatenating the first and last name.
-->
<xsl:function name="psf:member-name">
  <!-- The member node. -->
  <xsl:param name="member"/>
  <xsl:value-of select="concat($member/firstname, ' ', $member/surname)"/>
</xsl:function>

<!--
  Returns a pretty version of the specifid title.
-->
<xsl:function name="psf:pretty-title">
  <!-- The member node. -->
  <xsl:param name="title"/>
  <xsl:value-of select="
    if (starts-with(translate($title, '1234567890', 'xxxxxxxxxx'), 'xxxx-xx-xx-')) then
      substring(translate($title, '_', ' '), 12)
    else
      translate($title, '_', ' ')
  "/>
</xsl:function>

<!--
  Decodes the specified URI.
-->
<xsl:function name="psf:decode-uri">
  <!-- The URI to decode. -->
  <xsl:param name="uri"/>
  <xsl:sequence select="dec:decode($uri, 'utf-8')"/>
</xsl:function>

<!--
  Encodes the specified URI.
-->
<xsl:function name="psf:encode-uri">
  <!-- The URI to decode. -->
  <xsl:param name="uri"/>
  <xsl:if test="string($uri) != ''">
    <xsl:sequence select="enc:encode($uri, 'utf-8')"/>
  </xsl:if>
</xsl:function>

<!--
  Returns the class corresponding to the specified content type.
-->
<xsl:function name="psf:contenttype-to-class">
  <!-- The content type. -->
  <xsl:param name="type"/>
  <xsl:choose>
    <xsl:when test="$type = 'File Attachment'">upload</xsl:when>
    <xsl:when test="$type = 'File Upload'">upload</xsl:when>
    <xsl:when test="contains($type, 'archive')">archived</xsl:when>
    <xsl:otherwise>comment</xsl:otherwise>
  </xsl:choose>
</xsl:function>  

<!--
  Return the document type from uri 
-->
<xsl:function name="psf:uri-to-documenttype">
  <xsl:param name="uri"/>
  <xsl:value-of select="psf:type-path-to-documenttype($uri/type, $uri/path)" />
</xsl:function>
<!--
  Return the document type from uri type and path
-->
<xsl:function name="psf:type-path-to-documenttype">
  <xsl:param name="type"/>
  <xsl:param name="path"/>
  <!-- The name of the file -->
	<xsl:variable name="ext" select="psf:get-file-extension($path)"/>
	<xsl:choose>
		<xsl:when test="$type='application/pdf' or $ext='pdf'">pdf</xsl:when>
		<xsl:when test="$type='application/msword' or $ext='doc'">word</xsl:when>
		<xsl:when test="$type='application/vnd.ms-excel' or $ext='xls'">excel</xsl:when>
		<xsl:when test="$type='text/html' or $ext='htm'">html</xsl:when>
		<xsl:when test="$type='text/html' or $ext='html'">html</xsl:when>
		<xsl:when test="$type='text/html' or $ext='pshtml'">html</xsl:when>
		<xsl:when test="$type='image/gif' or $ext='gif'">graphic</xsl:when>
		<xsl:when test="$type='image/jpeg' or $ext='jpeg'">graphic</xsl:when>
		<xsl:when test="$type='image/png' or $ext='png'">graphic</xsl:when>
		<xsl:when test="$type='application/zip' or $ext='zip'">zip</xsl:when>
		<xsl:when test="$type='application/java-archive' or $ext='jar'">zip</xsl:when>
		<xsl:when test="$type='text/xml' or $ext='xml'">xml</xsl:when>
		<xsl:when test="$type='text/plain' or $ext='txt'">text</xsl:when>
		<xsl:otherwise>generic</xsl:otherwise>
	</xsl:choose>	
</xsl:function>

<!--
  Get file extension
-->
<xsl:function name="psf:get-file-extension">
  <!--  The name of a file -->
	<xsl:param name="name"/>
  <xsl:value-of select="
    if (contains($name, '.'))
    then
      psf:get-file-extension(substring-after($name, '.'))
    else
      lower-case($name)
  "/>
</xsl:function>
  
<!--
  Default to int
-->
<xsl:function name="psf:default-to-int">
	<!-- The name of the value -->
	<xsl:param name="value"/>
	<!-- The default value -->
  <xsl:param name="default" as="xs:integer"/>
  <xsl:value-of select="if ($value) then $value else $default"/>
</xsl:function>

<!--
  Formats the date component of the specified element.
  Assumes date is in the format yyyyMMdd, if the date is '0', returns '-'
  
  @return the formatted date as d MMM yy
-->
<xsl:function name="psf:format-modified-date">
  <!-- The date to format -->
	<xsl:param name="date"/>
  <xsl:value-of select="if ($date = '0') then '-' else
    concat(substring($date, 7, 2), ' ', psf:to-MMM(number(substring($date, 5, 2))), ' ', substring($date, 1, 4))"/>
</xsl:function>

<!--
  Formats the date component of the specified element.
  Assumes date is in the format yyyy-MM-ddTHH:mm:ss+XX:yy
  where XX:yy is the timezone offset in hours:minutes
  
  @return the formatted date as d MMM yy
-->
<xsl:function name="psf:format-date">
  <!-- The date to format -->
	<xsl:param name="date"/>
  <xsl:value-of select="concat(substring($date, 9, 2), ' ', psf:to-MMM(number(substring($date, 6, 2))), ' ', substring($date, 3, 2))"/>
</xsl:function>

<!--
  Formats the date component of the specified element using the ISO standard.
  Assumes date is in the format yyyy-MM-ddTHH:mm:ss+XX:yy
  where XX:yy is the timezone offset in hours:minutes
  
  @return the formatted date as yyyy-MM-dd
-->
<xsl:function name="psf:to-isodate">
  <!-- The date to format -->
	<xsl:param name="date"/>
  <xsl:value-of select="substring($date, 1, 10)"/>
</xsl:function>

<!--
  Formats the time component of the specified element.
  Assumes date is in the format yyyy-MM-ddTHH:mm:ss+XX:yy
  where XX:yy is the timezone offset in hours:minutes
  
  @return the formatted time as HH:mm (AM|PM)
-->
<xsl:function name="psf:format-time">
  <!-- The time to format -->
	<xsl:param name="datetime"/>
	<xsl:variable name="time">
		<xsl:variable name="time24" select="substring-after($datetime, 'T')" />
		<xsl:variable name="hour"   select="number(substring($time24,1,2))" />
		<xsl:variable name="minute" select="substring($time24,3,3)" />
    <xsl:value-of select="if ($hour = 0) then 12 else if ($hour > 12) then ($hour - 12) else $hour"/>
		<xsl:value-of select="$minute" />
    <xsl:value-of select="if ($hour >= 12) then ' PM' else ' AM'"/>
	</xsl:variable>
  <xsl:value-of select="$time"/>
</xsl:function>

<!--
  Formats the time component of the specified element.
  Assumes date is in the format yyyy-MM-ddTHH:mm:ss+XX:yy
  where XX:yy is the timezone offset in hours:minutes
  
  @return the formatted time as HHmmss
-->
<xsl:function name="psf:to-numeric-time">
  <!-- The time to format -->
	<xsl:param name="datetime"/>
  <xsl:value-of select="replace(substring-before(substring-after($datetime, 'T'), '+'),':', '')"/>
</xsl:function>

<!--
  Formats the date and time components of the current element.
  Assumes date is in the format yyyy-MM-ddTHH:mm:ss+XX:yy
  where XX:yy is the timezone offset in hours:minutes
  
  @return the formatted date as d MMM yy
-->
<xsl:function name="psf:format-datetime">
  <!-- The time to format -->
	<xsl:param name="datetime"/>
  <xsl:value-of select="concat(psf:format-date($datetime), ' ' , psf:format-time($datetime))"/>
</xsl:function>

<!--
  Returns the months as a three letter month name.
  
  @return The month formatted as MMM
-->
<xsl:function name="psf:to-MMM">
  <!-- The month as an integer. -->
  <xsl:param name="month"/>
  <xsl:choose>
		<xsl:when test="$month=1">Jan</xsl:when>
		<xsl:when test="$month=2">Feb</xsl:when>
		<xsl:when test="$month=3">Mar</xsl:when>
		<xsl:when test="$month=4">Apr</xsl:when>
		<xsl:when test="$month=5">May</xsl:when>
		<xsl:when test="$month=6">Jun</xsl:when>
		<xsl:when test="$month=7">Jul</xsl:when>
		<xsl:when test="$month=8">Aug</xsl:when>
		<xsl:when test="$month=9">Sep</xsl:when>
		<xsl:when test="$month=10">Oct</xsl:when>
		<xsl:when test="$month=11">Nov</xsl:when>
		<xsl:when test="$month=12">Dec</xsl:when>
	</xsl:choose>
</xsl:function>

<!--
  Returns the display name for a GroupURI

  @return the name
-->
<xsl:function name="psf:get-guri-path">
  <!-- The GroupURI node -->
  <xsl:param name="guri" />
  <xsl:choose>
    <xsl:when test="ends-with($guri/path, '/*')">
      <xsl:value-of select="substring($guri/path, 0, string-length($guri/path)-1)" />
    </xsl:when>
    <xsl:when test="ends-with($guri/path, '/')">
      <xsl:value-of select="substring($guri/path, 0, string-length($guri/path))" />
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="$guri/path" />
    </xsl:otherwise>
  </xsl:choose>
</xsl:function>
<!--
  Returns the display name for a URI

  @return the name
-->
<xsl:function name="psf:get-uri-name">
  <!-- The URI node -->
	<xsl:param name="uri" />
	<xsl:choose>
		<xsl:when test="$uri/displayname">
			<xsl:value-of select="$uri/displayname" />
		</xsl:when>
		<xsl:when test="$uri/usertitle">
			<xsl:value-of select="$uri/usertitle" />
		</xsl:when>
		<xsl:otherwise>
			<xsl:value-of select="psf:filename($uri/path)" />
		</xsl:otherwise>
	</xsl:choose>
</xsl:function>

<!--
  Gets a filname from the end of a full path.
  
  @return the filename
-->
<xsl:function name="psf:filename">
  <!-- The path for the file -->
	<xsl:param name="path"/>
  <xsl:value-of select="
    if (not($path))
      then 'undefined'
    else if (contains($path, '/'))
      then
        psf:filename(substring-after($path, '/'))
      else
        dec:decode($path,'UTF-8')
   "/>
</xsl:function>

<!--
  Shortens text when they are too long by working backwards and finding the
  first space that is available from the max length.
  
  @return the shortened text
-->
<xsl:function name="psf:shorten-text">
  <!-- The text to be shorted -->
	<xsl:param name="text"/>
	<!-- The maximum possible length for the text -->
	<xsl:param name="max-length"/>
  <xsl:value-of select="
    if (not($text))
      then ''
    else if (string-length($text) gt $max-length)
      then
        concat(substring($text, 1, $max-length), '...')
      else
        $text
   "/>
</xsl:function>

<!--
  Generates the Layout-style path URL
-->
<xsl:function name="psf:to-document-path">
  <!-- PageSeeder site prefix -->
  <xsl:param name="site-prefix"/>
  <!-- Current group name -->
  <xsl:param name="group-name"/>
  <!-- URI path for document -->
  <xsl:param name="ps-path"/>
  <!-- prefix of files in the same group, eg. ps/my/group -->
  <xsl:variable name="group-prefix" select="concat($site-prefix, '/' ,translate($group-name, '-', '/'))"/>
  <xsl:value-of select="
    if (starts-with($ps-path, $group-prefix))
      then
        concat('/path', substring($ps-path, string-length($group-prefix)+1))
      else 
    if (starts-with($ps-path, $site-prefix))
      then
        concat('/fullpath', substring($ps-path, string-length($site-prefix)+1))
      else
        concat('/filter', $ps-path)
  "/>
</xsl:function>

<!--
  Generates the Layout-style XRef URL.
  If href starts with # just return href
-->
<xsl:function name="psf:to-xref-path">
  <!-- PageSeeder site prefix -->
  <xsl:param name="site-prefix"/>
  <!-- current group name -->
  <xsl:param name="group-name"/>
  <!-- href attribute from xref -->
  <xsl:param name="href"/>
  
  <xsl:value-of select="psf:to-xref-page-path($site-prefix, $group-name, $href, false())"/>

</xsl:function>

<!--
  Generates the Layout-style XRef URL.
  If href starts with # just return href
-->
<xsl:function name="psf:to-xref-page-path">
  <!-- PageSeeder Site prefix -->
  <xsl:param name="site-prefix"/>
  <!-- Group name -->
  <xsl:param name="group-name"/>
  <!-- href attribute from xref -->
  <xsl:param name="href"/>
  <!-- if true use /fullpage in generated href -->
  <xsl:param name="fullpage"/>
  
  <xsl:choose>
  <xsl:when test="starts-with($href,'#')">
    <xsl:value-of select="$href"/>
  </xsl:when>
  <xsl:otherwise>
  	<xsl:variable name="page-prefix" select="if ($fullpage) then '/fullpage/' else '/page/'"/>
    <xsl:value-of select="concat($site-prefix,$page-prefix,$group-name,psf:to-document-path($site-prefix, $group-name, $href))"/>
  </xsl:otherwise>
  </xsl:choose>
</xsl:function>

<!-- CONTENT FILTERS ========================================================================== -->

<!--
  The PS Filter Period block
-->
<xsl:template name="psf:filter-period">
  <!-- PageSeeder Site prefix -->
<xsl:param name="site-prefix" select="$wo-constant.siteprefix"/>
	<!-- Group name -->
<xsl:param name="group-name" select="$group.name"/>
	<!-- Page path -->
<xsl:param name="page-path" select="$page.path"/>
	<!-- Page query string -->
<xsl:param name="page-query-string" select="$page.query-string"/>
	<!-- Current history -->
<xsl:param name="current-history"/>
<xsl:variable name="url" select="$page-path"/>
<span class="filter-title">Display</span>
<div id="ps-filter-period" class="filter">
  <label><xsl:value-of select="concat(if($current-history = 'all') then '' else 'Last ', $current-history, ' days')"/></label>
  <ul>
    <li><span class="period{if ($current-history = '7')   then ' selected' else ''}"><a href="{$url}?{psf:replace-param($page-query-string, 'history', '7')}">Last 7 days</a></span></li>
    <li><span class="period{if ($current-history = '30')  then ' selected' else ''}"><a href="{$url}?{psf:replace-param($page-query-string, 'history', '30')}">Last 30 days</a></span></li>
    <li><span class="period{if ($current-history = '90')  then ' selected' else ''}"><a href="{$url}?{psf:replace-param($page-query-string, 'history', '90')}">Last 90 days</a></span></li>
    <li><span class="period{if ($current-history = 'all') then ' selected' else ''}"><a href="{$url}?{psf:replace-param($page-query-string, 'history', 'all')}">All days</a></span></li>
  </ul>
</div>
</xsl:template>
	
<!--
  Returns whether the user has the requested permission.
-->
<xsl:function name="psf:has-permission">
  <!-- The name of the permission. -->
  <xsl:param name="permission"/>
  <xsl:sequence select="if ($permission = 'true') then true() else false()"/>
</xsl:function>
	
<!--
  Returns the sequence of uri id for this uri
-->
<xsl:function name="psf:getStateFor">
  <!-- Status -->
  <xsl:param name="state" />
  <!-- URI -->
  <xsl:param name="uri" />
  <!-- URI type (e.g. 'U' or 'P') -->
  <xsl:param name="type" />
  <xsl:variable name="uid" select="concat($type, $uri/id, '.')"/>
  <xsl:value-of select="
if (contains($state, $uid))
  then
    replace($state, concat($type, $uri/id, '\.'), '')
  else
    concat($state, $uid)
"/>
</xsl:function>
  
<!--
  Returns true if this field should be ignored in criteria search
-->
<xsl:function name="psf:is-ignorable-field">
  <!-- Field to be determined -->
  <xsl:param name="field" />
  <xsl:value-of select="(starts-with(string($field), 'ps') and not(string($field) = 'psdocid') and not(string($field) = 'psurifolder'))
                         or string($field) = 'fragment' or string($field) = 'idxid' or string($field) = 'indexed'" />
</xsl:function>
  
  
<!--
  Returns a printable version of the search field
  i.e.: last_edited_date -> Last edited date
-->
<xsl:function name="psf:pretty-search-field">
  <!-- Field to be determined -->
  <xsl:param name="field" />
  <xsl:value-of select="if ($field = 'psdocid') then 'Document ID' else if ($field = 'psurifolder') then 'Folder' else
                        concat(upper-case(substring($field, 1, 1)), replace(substring($field, 2), '_', ' '))" />
</xsl:function>
  
<!--
  Returns a printable version of the search term
  i.e.: last_edited_date -> Last edited date
-->
<xsl:function name="psf:pretty-search-term">
  <!-- Field to be determined -->
  <xsl:param name="field" />
  <!-- Search term -->
  <xsl:param name="term" />
  <xsl:value-of select="if (contains($field, 'date')) then psf:format-modified-date($term) else $term" />
</xsl:function>

<!-- Encode sourcetitle for URI -->
<xsl:function name="psf:encode-sourcetitle-for-uri">
  <!-- Source title -->
  <xsl:param name="string" />
  <xsl:value-of select="encode-for-uri(replace(replace($string, '\n', ''), '[\s&#xa0;]', ' '))" />
</xsl:function>

  
<!--
  Returns a printable version of the group property
  i.e.: notifyFromAddress -> Notify From Address
-->
<xsl:function name="psf:pretty-group-property">
  <!-- Group property -->
  <xsl:param name="property" />
  <xsl:value-of select="normalize-space(concat(upper-case(substring($property, 1, 1)), string-join(
            for $i in 2 to string-length($property) return
              if (matches(substring($property, $i, 1), '[A-Z]')) then concat(' ', substring($property, $i, 1))
              else substring($property, $i, 1), '')))" />
</xsl:function>

</xsl:stylesheet>