<?xml version="1.0" encoding="UTF-8"?>
<?xar Schematron?>
<sch:schema xmlns:sch="http://purl.oclc.org/dsdl/schematron"
            queryBinding="xslt2" >
        <sch:title>Validation for importing docx as PSXML</sch:title>
        <sch:ns prefix="w" uri="http://schemas.openxmlformats.org/wordprocessingml/2006/main"/>
        <sch:ns prefix="r" uri="http://schemas.openxmlformats.org/officeDocument/2006/relationships"/>
        <sch:ns prefix="ve" uri="http://schemas.openxmlformats.org/markup-compatibility/2006" />
        <sch:ns prefix="o" uri="urn:schemas-microsoft-com:office:office"/>
        <sch:ns prefix="r" uri="http://schemas.openxmlformats.org/officeDocument/2006/relationships"/> 
        <sch:ns prefix="m" uri="http://schemas.openxmlformats.org/officeDocument/2006/math" /> 
        <sch:ns prefix="v" uri="urn:schemas-microsoft-com:vml" />
        <sch:ns prefix="wp" uri="http://schemas.openxmlformats.org/drawingml/2006/wordprocessingDrawing" />
        <sch:ns prefix="w10" uri="urn:schemas-microsoft-com:office:word" />
        <sch:ns prefix="w" uri="http://schemas.openxmlformats.org/wordprocessingml/2006/main"/>
        <sch:ns prefix="wne" uri="http://schemas.microsoft.com/office/word/2006/wordml"/>
        
    <!-- 
        ==========================================================================
        This schema validates docx for importing as PSXML.
        
        @author Christine Feng
        @author William Liem
        @author Rick Jellife
        @author Philip Rutherford
        @version 23 March 2011
    
        Copyright (C) 2011 Weborganic Systems Pty. Ltd.
        ==========================================================================
     -->
    
    <sch:pattern id="Elements">
        
        <sch:rule context="w:tbl[descendant::w:tbl]">
            <sch:assert test="false()">[FATAL]:A table inside a table is not supported</sch:assert>
        </sch:rule>
                
                
        <!-- 
              Warning: Tables found
              If tables are found, an entry should be added to the error report regarding widths - the error is non-fatal.
        -->
        <sch:rule context="w:tbl">
            <sch:report test="true()">Table cell widths will be ignored:</sch:report>
        </sch:rule>
        
        
        <!--
             Invalid headings - these errors are non-fatal.
             Currently the following validation is performed:
              - cannot have empty content (stripped out)
              - cannot contain an image (image is stripped out)
              - cannot be located in a table (turned to a paragraph) 
        -->
        <sch:rule context="w:p[matches(w:pPr/w:pStyle/@w:val, 'Heading(1|2|3|4|5|6)')]" >
          <sch:assert test="string-join(w:r//text(), '') != ''">Empty headings will be ignored: Empty <sch:value-of select="w:pPr/w:pStyle/@w:val"/> found</sch:assert>
          <sch:assert test="empty(.//w:drawing | .//w:pict)">Images in headings will be stripped out:<sch:value-of select="w:r/w:t"/></sch:assert>
          <sch:assert test="empty(ancestor::w:tbl)">Headings in table will be imported as paragraphs:<sch:value-of select="w:r/w:t"/></sch:assert>
        </sch:rule>
    </sch:pattern>
</sch:schema>
