/**
 * Behaviour of the project files page
 * 
 * @author Christophe Lauret
 * @version 25 October 2010
 */

// Declare all global dependencies for JSLint
/*global PAGESEEDER, $ */

// Defines and invokes an anonymous function to create a closure.
$(document).ready(function() {

  // Our container to control the tree
  var $treecontrol = $('<div class="treecontrol"></div>');
  $('#project-files').before($treecontrol);

  // Expand / Collapse Buttons
  var $expand = $('<a class="button tiny ps-gradient expand-all" href="#">Expand all</a>').click(function() {
    $('#ps-content-project .ps-tree-node .off').toggleClass('off', false).toggleClass('on', true);
    $('#ps-content-project .ps-tree-node-content').toggle(true);
    return false;
  });
  var $collapse = $('<a class="button tiny ps-gradient collapse-all" href="#">Collapse all</a>').click(function() {
    $('#ps-content-project .ps-tree-node .on').toggleClass('off', true).toggleClass('on', false);
    $('#ps-content-project .ps-tree-node-content').toggle(false);
    return false;
  });
  $treecontrol.append($expand).append($collapse);

  // Export
  var $export = $('<a class="button tiny ps-gradient export" href="#">Export</a>').click(function() {
    $.get(PublisherContext+'/admin/resources/export.xml', {'project': PublisherProject},
      function (data) {
        var filename = $(data).find('export').attr('file');
        var project = $(data).find('export').attr('project');
        window.location = PublisherContext+'/admin/resource?location=/WEB-INF/template/'+project+'/.export/'+filename+'&download=true';
      });
    return false;
  });
  $treecontrol.append($export);

  // File dropping supported
  if ($.fn.psDropFile.supported) {

    // Where the files are dropped
    var $zone = $('<div id="dropzone"></div>');
    $('#project-files').before($zone);
    $zone.psDropFile({
      url: PublisherContext+'/admin/upload?&project='+PublisherProject,
      method: 'PUT',
      hoverClass: 'hover',
      uploadRateRefreshTime : 1000
    });

    // Invoked when the upload for given file has been started
    $.fn.psDropFile.uploadStarted = function(fileIndex, file) {
      var $li = $('<li class="ps-tree-node">');
      var $info = $('<div id="uploadinfo-'+fileIndex+'" class="ps-tree-node-file leaf"><a href="#" class="ps-tree-node-title"><span class="file"><i class="psi psi-16 psi-mediatype"/>&nbsp;'+(file.name || file.fileName)+'&nbsp;<span class="size">0 bytes</span></span></a></div>');
      $li.append($info);
      var $bar = $('<div id="progressbar-'+fileIndex+'"></div>');
      $info.append($bar);
      $bar.progressbar({value: 0});
      $('#dropbox ul').append($li);
    };

    // invoked when the upload for given file has been finished
    $.fn.psDropFile.uploadFinished = function(fileIndex, file, time, xhr) {
      // Remove the progress bar
      $('#progressbar-'+fileIndex).progressbar('value', 100).remove();
      // No error we upload the node
      if (xhr.status < 400) {
        var $f = $(xhr.responseXML).find('file');
        var media = $f.attr('media-type').replace(/\W|\_/g, '-');
        var zip = $f.attr('media-type') === 'application/zip';
        var $node = $('#uploadinfo-'+fileIndex);
        var $actions = $('<span class="actions"><a title="Delete file" class="delete" href="#delete">X</a><a title="Rename file" class="rename" href="#rename">R</a><a title="Show file history" class="history" href="#history">H</a>'+(zip? '<a title="Import" class="import" href="#import">I</a>':'')+'</span>');
        $node.parent().attr('data-location', $f.attr('path'));
        $node.find('.ps-node-title').attr('href', PublisherContext+'/admin/resources/get.html?&location='+$f.attr('path'));
        $node.draggable(draggableFileOptions).addClass(media);
        // update loaded size
        var size = $f.attr('length') > 1000? Math.round($f.attr('length') / 1000)+' kB' : $f.attr('length')+' bytes';
        $node.find(' .size').text(size);
        $node.append($actions).append('<span class="modified">A moment ago</span>');
        $node.find('.delete').click(triggerDelete);
        $node.find('.rename').click(triggerRename);
        $node.find('.history').click(viewFileHistory);
        $node.find('.import').click(triggerImport);
      // Something happened
      } else {
        var $delete = $('<span class="actions"><a title="Delete file" class="delete" href="#delete">X</a></span>');
        $delete.click(function() { $(this).closest('li').hide('puff'); });
        $('#uploadinfo-'+fileIndex).append($delete).append('<span class="modified">Error '+xhr.status+'</span>');
      }
    };

    // invoked when the progress for given file has changed
    $.fn.psDropFile.fileUploadProgressUpdated = function(fileIndex, file, progress) {
      if (progress.lengthComputable) {
        // update progress bar
        var percentage = Math.round((progress.loaded * 100) / progress.total);
        $('#progressbar-'+fileIndex).progressbar('value', percentage);
        // update loaded size
        var size = progress.loaded > 1000? Math.round(progress.loaded / 1000)+' kB' : progress.loaded+' bytes';
        $('#uploadinfo-'+fileIndex+' .size').text(size);
      }
    };

  }

  /**
   * Create a new file node in the tree.
   * @param $f The jQuery file XML from the service that created the file.
   */
  var newFile = function ($f) {
    var file = {
      location : $f.attr('path'),
      name : $f.attr('name'),
      type : $f.attr('type'),
      size : 0,
      media : $f.attr('media-type'),
      modified : 'A moment ago'
    };
    var $li = $('<li class="ps-tree-node"></li>');
    $li.attr('data-location', file.location);
    var actions, $node;
    // A new folder
    if (file.type === 'folder') {
      actions = '<a title="Delete folder" class="delete" href="#delete"><span>X</span></a>'+
                '<a title="Upload file into folder" class="upload" href="#upload"><span>U</span></a>'+
                '<a title="Create file/folder in folder" class="create" href="#create"><span>C</span></a>';
      $node = $('<div class="ps-tree-node-folder on ps-droppable empty-folder"><a class="toggle" title="Expand / Collapse" href="#"/><span class="ps-tree-node-title"></span><span class="actions">'+actions+'</span></div>');
      $node.find('.ps-tree-node-title').text(file.name);
      $node.find('.upload').click(triggerUpload);
      $node.find('.create').click(triggerCreate);
      $node.droppable(droppableFileOptions);
    // A new file
    } else if (file.type === 'file') {
      actions = '<a title="Delete file" class="delete" href="#delete"><span>X</span></a>'+
                '<a title="Rename file" class="rename" href="#rename"><span>R</span></a>'+
                '<a title="Show file history" class="history" href="#history"><span>H</span></a>';
      $node = $('<div class="ps-tree-node-'+file.type+' leaf">'+
                  '<a href="'+PublisherContext+'/admin/resources/get.html?location='+encodeURI(file.location)+'" class="ps-tree-node-title">'+
                    '<span class="file"><i class="psi psi-16 psi-mediatype '+mediatypeToIcon(file.media)+'"/>&nbsp;'+file.name+'&nbsp;<span class="size">'+file.size+' bytes</span></span>'+
                  '</a>'+
                  '<span class="actions">'+actions+'</span>'+
                '</div>');
      $node.find('.rename').click(triggerRename);
      $node.find('.history').click(viewFileHistory);
      $node.draggable(draggableFileOptions);
    }
    $node.find('.delete').click(triggerDelete);
    $li.append($node);
    if (file.type === 'folder') { $li.append('<div class="ps-tree-node-content"><ul></ul></div>'); }
    return $li;
  };

  /**
   * Find an icon class from a file's mediatype.
   * 
   * @param mediatype the file's mediatype
   * 
   * @returns the icon class
   */
  function mediatypeToIcon(mediatype) {
    if (!mediatype) return '';
    // PageSeeder types
    if (mediatype === 'application/vnd.pageseeder.document.standard+xml') return 'psstandard';
    if (mediatype === 'application/vnd.pageseeder.psml+xml') return 'psml';
    // Specific MIME
    if (mediatype === 'application/pdf') return 'pdf';
    if (/^application\/vnd.ms-word/.test(mediatype) || /^application\/msword/.test(mediatype)) return 'ms-word';
    if (/^application\/vnd.ms-excel/.test(mediatype)) return 'ms-excel';
    if (/^application\/vnd.ms-powerpoint/.test(mediatype)) return 'ms-powerpoint';
    if (mediatype === 'application/vnd.openxmlformats-officedocument.wordprocessingml.document') return 'openxml-word';
    if (mediatype === 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet') return 'openxml-excel';
    if (mediatype === 'application/vnd.openxmlformats-officedocument.presentationml.presentation') return 'openxml-powerpoint';
    if (mediatype === 'text/html') return 'html';
    if (mediatype === 'text/css') return 'css';
    if (mediatype === 'text/properties') return 'properties';
    if (mediatype === 'application/javascript' || mediatype === 'application/json') return 'javascript';
    if (mediatype === 'application/zip' ) return 'compressed';
    if (mediatype === 'application/x-tar') return 'compressed';
    if (mediatype === 'application/x-gzip') return 'compressed';
    if (mediatype === 'application/java-archive') return 'compressed';
    if (mediatype === 'application/xml-dtd' || mediatype === 'application/schematron+xml') return 'xml-validate';
    if (mediatype === 'application/xslt+xml') return 'xml-transform';
    // Main categories
    if (/\.xml$/.test(mediatype)) return 'xml';
    if (/^text\//.test(mediatype)) return 'text';
    if (/^image\//.test(mediatype)) return 'image';
    if (/^audio\//.test(mediatype)) return 'audio';
    if (/^video\//.test(mediatype)) return 'video';
    // Fall back on
    return mediatype.replace(/[^a-z0-9]/i, '-');
  };

  // Import =======================================================================================

  /**
   * Sends a request to import the file at the specified location.
   */
  var importPack = function (pack) {
    $.ajax({
      type: "POST",
      url: PublisherContext+'/admin/resources/import.xml',
      data: {'pack' : pack},
      success: function(data) {
        window.location.reload();
      },
      error: function(xhr, status, error){
        $.pspublisher.error($(xhr.responseXML).find('message').text());
      }
    });
  };

  /**
   * Prompts the user to ask whether to unpack the content of the kit pack.
   */
  var triggerImport = function() {
    var $button = $(this);
    var $node = $button.closest('.ps-tree-node');
    var $confirm = $('<div></div>');
    $.pspublisher.confirm('Are you sure you would like to unpack and import this file?', 'Unpack file', function() {
      importPack($node.attr('data-location'));
    });
    return false;
  };

  // Bind import action
  $('.actions .import').click(triggerImport);

  // Upload =======================================================================================

  /**
   * Uploads the file by submitting the form.
   */
  var uploadFile = function (form, $dialog) {
    var $form = $(form);
    var targetId = 'upload-xyz';
	  var $iframe = $('<iframe id="'+targetId+'" name="'+targetId+'" style="display: none;"></iframe>');
	  $form.attr('target', targetId);
	  $form.after($iframe);
	  $iframe.load(function() {
	    $form.find('input[type=file]').hide();
	    $.pspublisher.dialog('File was successfully uploaded!', 'Success', function() { window.location.reload(); }); 
	  });
	  form.submit();
  };

  /**
   * Displays upload form.
   */
  var triggerUpload = function() {
    var $button = $(this);
    var $node = $button.closest('.ps-tree-node');
    var location = $node.attr('data-location');
    var uploadServlet = PublisherContext+'/admin/upload';
    var $form = $('<form id="resource-upload-form" action="'+uploadServlet+'" enctype="multipart/form-data" method="POST"></form>');
    $form.append('<p>Select the file to upload into <b>'+location.substring(location.lastIndexOf('/')+1)+'</b></p>');
    $form.append('<input name="location" type="hidden" readonly="readonly" value="'+location+'"/></input>');
    $form.append('<input name="file-1" type="file" /></input>');
    $.pspublisher.dialog($form, 'Upload file', function() {
      uploadFile($form.get(0), $(this));
      $form.get(0).submit();
    }, '<button class="button radius primary close">upload</button>');
    return false;
  };

  // Actions on folder
  $('.actions .upload').click(triggerUpload);

  // Create =======================================================================================
  
  /**
   * Sends a request to create the file at the specified location.
   */
  var createFile = function (location, name) {
	  $.ajax({
      type: "POST",
	    url: PublisherContext+'/admin/resources/create.xml',
	    data: {'location' : location+'/'+name},
      complete: function(xhr, textStatus) {
        if (xhr.status >= 400) {
          $.pspublisher.error($(xhr.responseXML).find('message').text());
        } else {
          var $f = $(xhr.responseXML).find('file');
          $('#project-files').find('li[data-location=\''+location+'\'] > .ps-tree-node-content > ul').append(newFile($f));
        }
      }
    });
  };

  /**
   * Triggers the creation of a new resource (opens dialog for details)
   */
  var triggerCreate = function () {
    var $button = $(this);
    var $node = $button.closest('.ps-tree-node');
    var $form = $('<form id="create-resource-dialog" onsubmit="return false;"></form>');
    $form.append('<p>Enter the name of the file/folder you would like to create:</p>');
    $form.append('<p><input name="name" type="text"></input></p>');
    var buttons = '<button class="button radius primary create">Create</button>'+
                  '<button class="button radius secondary close">Cancel</button>';
    var $box = $.pspublisher.dialog($form, 'Create file', 'tiny', null, buttons);
    $form.find('input').keyup(function(event) {
      if (event.keyCode == '13') {
        event.preventDefault();
        $box.find('button.create').click();
      }
    }).focus();
    $box.find('button.create').click(function() {
      var name = $form.find('input').val();
      if (name.match(/\w[\w\.]+/m)) {
        createFile($node.attr('data-location'), name);
        $box.foundation('reveal', 'close');
      } else {
        $form.append('<p>Invalid file name!</p>');
      }
      return false;
    });
    return false;
  };

  // Actions folder
  $('.actions .create').click(triggerCreate);

  // Delete ======================================================================================

  /**
   * Sends a request to delete the file at the specified location.
   */
  var deleteFile = function (location, $node) {
	  $.ajax({
      type: "POST",
	    url: PublisherContext+'/admin/resources/delete.xml',
	    data: {'location' : location},
      success: function(data) { $node.hide('puff'); },
      error: function(xhr, status, error) {
        var msg = $(xhr.responseXML).find('message').text();
        $.pspublisher.error(!msg ? xhr.statusText : msg);
      }
    });
  };

  /**
   * Triggers the deletion of a resource (opens dialog for confirmation)
   */
  var triggerDelete = function() {
    var $button = $(this);
    var $node = $button.closest('.ps-tree-node');
    var $content = $('<div id="delete-resource-dialog">');
    $content.append('<p>Are you sure you want to delete this file?</p>');
    $content.append('<p><b></b></p>');
    $content.find('b').text($node.attr('data-location'));
    $.pspublisher.confirm($content, 'Delete file?', 'tiny', function($box) {
      deleteFile($node.attr('data-location'), $node);
      $box.foundation('reveal', 'close');
    });
    return false;
  };

  // Actions on files and folder
  $('.actions .delete').click(triggerDelete);

  // Rename =======================================================================================

  /**
   * Sends a request to delete the file at the specified location.
   * 
   * @param from  the location of the resource to rename
   * @param to    the new location of the resource
   * @param $node the 'li.ps-tree-node' being renamed 
   */
  var renameFile = function (from, to, $node) {
    $.ajax({
      type: "POST",
      url: PublisherContext+'/admin/resources/move.xml',
      data: {'location' : from, 'to': to },
      success: function(data) {
        var $f = $(data).find('file');
        var location = $f.attr('path');
        var name = location.substring(location.lastIndexOf('/')+1);
        var size = $node.find('span.file .size').text();
        $node.attr('data-location', location);
        $node.find('.ps-tree-node-title').attr('href', PublisherContext+'/admin/resources/get.html?location='+encodeURI(location));
        $node.find('.actions a.download').attr('href', PublisherContext+'/admin/resource?download=true&location='+encodeURI(location));
        var icon = $('<i>').attr('class', $node.find('span.file i').attr('class'));
        $node.find('span.file').text(' '+name+' ').prepend(icon).append('<span class="size">'+size+'</span>');
      },
      error: function(xhr, status, error){
        $.pspublisher.error($(xhr.responseXML).find('message').text());
      }
    });
  };

  // Actions on files and folder
  var triggerRename = function () {
    var $button = $(this);
    var $node = $button.closest('.ps-tree-node');
    var location = $node.attr('data-location');
    var folder = location.substring(0, location.lastIndexOf('/')+1);
    var name = location.substring(location.lastIndexOf('/')+1);
    var $form = $('<form id="resource-rename-form" action="#" onsubmit="return false;"></form>');
    $form.append('<p>What would you like to rename <b>'+name+'</b> to?</p>');
    $form.find('b').text(name);
    $form.append('<input name="to" type="text" value="'+name+'"/></input>');
    var buttons = '<button class="button radius primary rename">Rename</button>'+
                  '<button class="button radius secondary close">Cancel</button>';
    var $box = $.pspublisher.dialog($form, 'Rename file', 'tiny', null, buttons);
    $form.find('input').focus().keyup(function(event) {
      if (event.keyCode == '13') {
        event.preventDefault();
        $box.find('button.rename').click();
      }
    });
    $box.find('button.rename').click(function() {3
      var newname = $form.find('input').val();
      if (newname.match(/\w[\w\.]+/m)) {
        renameFile($node.attr('data-location'), folder+newname, $node);
        $box.foundation('reveal', 'close');
      } else {
        $form.append('<p>Invalid file name!</p>');
      }
      return false;
    });
    return false;
  };

  $('.actions .rename').click(triggerRename);

  // Move =========================================================================================

  /**
   * Sends a request to delete the file at the specified location.
   * 
   * @param from   the location of the file to move
   * @param to     the location of the destincation folder
   * @param file   the '.ps-tree-node-file' jquery object
   * @param folder the '.ps-tree-node-folder' jquery object
   */
  var moveFile = function (from, to, file, folder) {
	  $.ajax({
      type: "POST",
	    url: PublisherContext+'/admin/resources/move.xml',
	    data: {'location' : from, 'to': to},
      success: function(data) {
        var xml = $(data);
        if (file && folder) {
          var location = xml.find('file').attr('path');
          $(file).parent().attr('data-location', location);
          $(folder).next().children('ul').append($(file).parent());
          $(folder).removeClass('empty-folder');
    	  }
      },
      error: function(xhr, status, error){
        $.pspublisher.error($(xhr.responseXML).find('message').text());
      }
    });
  };

  var triggerMove = function() {
    var $button = $(this);
    var $node = $button.closest('.ps-tree-node');
    var $form = $('<form id="resource-move-form" action="'+$button.attr('href')+'" method="POST"></form>');
    $form.append('<p>Where would you like to move <b>'+$node.attr('data-location')+'</b>?</p>');
    $form.append('<input name="to" type="text" value="'+$node.attr('data-location')+'"/></input>');
    $.pspublisher.dialog($form, 'Move file', 'tiny', function() {
      moveFile($node.attr('data-location'), $form.find('input').val());
    });
    return false;
  };

  // Actions on files and folder
  $('.actions .move').click(triggerMove);
  
  // Draggable files
  var draggableFileOptions = {
    cancel: ".actions",
    revert: "invalid",
    containment: "#project-files",
    helper: "clone",
    cursor: "move"
  };
  $(".ps-tree-node-file").draggable(draggableFileOptions);

  // Droppable files (only files for now)
  var droppableFileOptions = {
    accept: ".ps-tree-node-file",
    activeClass: "filedrop",
    hoverClass: "drophover",
    drop: function (event, ui) {
      var from = $(ui.draggable).parent().attr('data-location');
      var to = $(this).parent().attr('data-location');
      moveFile(from, to, $(ui.draggable), $(this));
    }
  };
  $(".ps-droppable").droppable(droppableFileOptions);

  // History ======================================================================================

  /**
   * Sends a request to delete the file at the specified location.
   */
  var viewFileHistory = function () {
    var $button = $(this);
    var $node = $button.closest('.ps-tree-node');
    var location = $node.attr('data-location');
    $.ajax({
      type: "GET",
      url: PublisherContext+'/admin/resources/history.xml',
      data: {'location' : location},
      success: function(data) {
        var $history = $('<div id="file-history"></div>');
        var fname = location.substring(location.lastIndexOf('/')+1);
        var $files = $(data).find('file');
        if (!$files.length) {
          $history.append('<p>No history found for <b>'+fname+'</b></p>');
        } else {
          var $table = $('<table>');
          $table.append('<tr><th class="revision">Revision</th><th class="size">Size</th><th class="modified">Last Modified</th></tr>');
          $history.append($table);
          $files.each(function() {
            var name = $(this).attr('name');
            var revision = name.substring(name.lastIndexOf('.r')+2);
            var row = '<tr><th class="revision">'+revision+'</th>';
            row += '<td class="size">'+$(this).attr('length')+' bytes</td>';
            row += '<td class="modified">'+new Date($(this).attr('modified')).toLocaleString()+'</td>';
            row += '</tr>';
            $table.append(row);
          });
        }
        $.pspublisher.dialog($history, 'File History for '+fname);
      },
      error: function(xhr, status, error){
        $.pspublisher.error($(xhr.responseXML).find('message').text());
      }
    });
    return false;
  };

  // Actions on files and folder
  $('.actions .history').click(viewFileHistory);
  
});
